/*
* The Skein-512 hash function
* (C) 2009 Jack Lloyd
*
* Distributed under the terms of the Botan license
*/

#ifndef BOTAN_SKEIN_512_H__
#define BOTAN_SKEIN_512_H__

#include <botan/secmem.h>
#include <botan/hash.h>
#include <string>

namespace Botan {

/**
* Skein-512, a SHA-3 candidate
*/
class BOTAN_DLL Skein_512 : public HashFunction
   {
   public:
      /**
      * @param output_bits the output size of Skein in bits
      * @param personalization is a string that will paramaterize the
      * hash output
      */
      Skein_512(u32bit output_bits = 512,
                const std::string& personalization = "");

      HashFunction* clone() const;
      std::string name() const;
      void clear();
   private:
      void add_data(const byte input[], u32bit length);
      void final_result(byte out[]);

      std::string personalization;
      u32bit output_bits;
      SecureVector<u64bit, 9> H;
      SecureVector<u64bit, 3> T;

      SecureVector<byte, 64> buffer;
      u32bit buf_pos;
   };

}

#endif
